@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -20,10 +20,9 @@ from __future__ import division, print_function, absolute_import
 from collections import namedtuple
 
 import hypothesis.internal.conjecture.utils as cu
-from hypothesis.control import assume
-from hypothesis.internal.compat import OrderedDict
+from hypothesis.internal.compat import OrderedDict, hbytes
 from hypothesis.searchstrategy.strategies import SearchStrategy, \
-    one_of_strategies, MappedSearchStrategy
+    MappedSearchStrategy, one_of_strategies
 
 
 class TupleStrategy(SearchStrategy):
@@ -41,7 +40,7 @@ class TupleStrategy(SearchStrategy):
         strategies = tuple(strategies)
         self.element_strategies = strategies
 
-    def validate(self):
+    def do_validate(self):
         for s in self.element_strategies:
             s.validate()
 
@@ -54,6 +53,9 @@ class TupleStrategy(SearchStrategy):
             tuple_string,
         )
 
+    def calc_has_reusable_values(self):
+        return all(e.has_reusable_values for e in self.element_strategies)
+
     def newtuple(self, xs):
         """Produce a new tuple of the correct type."""
         return tuple(xs)
@@ -63,6 +65,12 @@ class TupleStrategy(SearchStrategy):
             data.draw(e) for e in self.element_strategies
         )
 
+    def calc_is_empty(self):
+        return any(e.is_empty for e in self.element_strategies)
+
+
+TERMINATOR = hbytes(b'\0')
+
 
 class ListStrategy(SearchStrategy):
 
@@ -76,10 +84,6 @@ class ListStrategy(SearchStrategy):
 
     """
 
-    Parameter = namedtuple(
-        'Parameter', ('child_parameter', 'average_length')
-    )
-
     def __init__(
         self,
         strategies, average_length=50.0, min_size=0, max_size=float('inf')
@@ -93,32 +97,18 @@ class ListStrategy(SearchStrategy):
         self.max_size = max_size or float('inf')
         self.element_strategy = one_of_strategies(strategies)
 
-    def validate(self):
+    def do_validate(self):
         self.element_strategy.validate()
 
     def do_draw(self, data):
-        if self.max_size == self.min_size:
-            return [
-                data.draw(self.element_strategy)
-                for _ in range(self.min_size)
-            ]
-
-        stopping_value = 1 - 1.0 / (1 + self.average_length)
+        elements = cu.many(
+            data,
+            min_size=self.min_size, max_size=self.max_size,
+            average_size=self.average_length
+        )
         result = []
-        while len(result) < self.max_size:
-            data.start_example()
-            more = cu.biased_coin(data, stopping_value)
-            if not more:
-                data.stop_example()
-                if len(result) < self.min_size:
-                    continue
-                else:
-                    break
-            value = data.draw(self.element_strategy)
-            data.stop_example()
-            result.append(value)
-        else:
-            cu.write(data, b'\0')
+        while elements.more():
+            result.append(data.draw(self.element_strategy))
         return result
 
     def __repr__(self):
@@ -145,7 +135,7 @@ class UniqueListStrategy(SearchStrategy):
         self.element_strategy = elements
         self.key = key
 
-    def validate(self):
+    def do_validate(self):
         self.element_strategy.validate()
 
     Parameter = namedtuple(
@@ -153,38 +143,23 @@ class UniqueListStrategy(SearchStrategy):
     )
 
     def do_draw(self, data):
+        elements = cu.many(
+            data,
+            min_size=self.min_size, max_size=self.max_size,
+            average_size=self.average_size
+        )
         seen = set()
         result = []
-        if self.max_size == self.min_size:
-            while len(result) < self.max_size:
-                v = data.draw(self.element_strategy)
-                k = self.key(v)
-                if k not in seen:
-                    result.append(v)
-                    seen.add(k)
-            return result
 
-        stopping_value = 1 - 1.0 / (1 + self.average_size)
-        duplicates = 0
-        while len(result) < self.max_size:
-            data.start_example()
-            if len(result) >= self.min_size:
-                more = cu.biased_coin(data, stopping_value)
-            else:
-                more = True
-            if not more:
-                data.stop_example()
-                break
+        while elements.more():
             value = data.draw(self.element_strategy)
-            data.stop_example()
             k = self.key(value)
             if k in seen:
-                duplicates += 1
-                assume(duplicates <= len(result))
-                continue
-            seen.add(k)
-            result.append(value)
-        assume(len(result) >= self.min_size)
+                elements.reject()
+            else:
+                seen.add(k)
+                result.append(value)
+        assert self.max_size >= len(result) >= self.min_size
         return result
 
 
